#!/usr/local/bin/python
# coding: latin-1

from xlwt.Workbook import Workbook
import xlwt

class SalvaXLS(object):
    def __init__(self, nomeDoArquivo):
        self.nomeDoArquivo = nomeDoArquivo
        self.arquivo = Workbook(encoding='latin-1')
        self.tabelaDeParametros = self.arquivo.add_sheet('Parmetros')
        self.tabelaDeResultados = self.arquivo.add_sheet('Resultados')

        self.defineTamanhoDasColunas()

        self.estiloNegrito = xlwt.easyxf('font: bold on')

    def defineTamanhoDasColunas(self):
        tabela = self.tabelaDeParametros
        tabela.col(0).width = 36 * 256
        tabela.col(1).width = 33 * 256
        tabela.col(2).width = 25 * 256
        tabela.col(3).width = 25 * 256
        tabela.col(4).width = 25 * 256
        tabela.col(5).width = 23 * 256

        tabela = self.tabelaDeResultados
        tabela.col(0).width = 12 * 256
        tabela.col(1).width = 22 * 256
        tabela.col(2).width = 18 * 256
        tabela.col(3).width = 9 * 256
        tabela.col(5).width = 8 * 256
        tabela.col(6).width = 8 * 256

    def salvaXLS(self, parametrosDoUsuario, leilao=None, historicoDosLeiloes=None):
        if self.nomeDoArquivo == None:
            return False

        self.salvaTabelaDeParametros(parametrosDoUsuario)

        if parametrosDoUsuario.numeroDeLeiloes == 1 and leilao != None:
            self.salvaTabelaDeResultadosParaUmLeilao(leilao)
        elif parametrosDoUsuario.numeroDeLeiloes > 1 and historicoDosLeiloes != None:
            self.salvaTabelaDeResultadosParaVariosLeiloes(historicoDosLeiloes)
        else:
            return False

        self.arquivo.save(self.nomeDoArquivo)

        return True

    def salvaTabelaDeResultadosParaUmLeilao(self, leilao):
        self.tabelaDeResultados.write(0, 0, "Participante", self.estiloNegrito)
        self.tabelaDeResultados.write(0, 1, "Valor para participante", self.estiloNegrito)
        self.tabelaDeResultados.write(0, 2, "Dinheiro disponvel", self.estiloNegrito)
        self.tabelaDeResultados.write(0, 3, "Perfil", self.estiloNegrito)

        for i in range(leilao.numeroDeParticipantes):
            self.tabelaDeResultados.write(i + 1, 0, i + 1)
            self.insereParticipante(leilao.participantes[i], i + 1)

        self.tabelaDeResultados.write(0, 5, "Autor", self.estiloNegrito)
        self.tabelaDeResultados.write(0, 6, "Oferta", self.estiloNegrito)

        for i in range(len(leilao.historicoDosLances)):
            self.tabelaDeResultados.write(i + 1, 5, leilao.autorDoLance[i] + 1)
            self.tabelaDeResultados.write(i + 1, 6, leilao.historicoDosLances[i])

    def salvaTabelaDeResultadosParaVariosLeiloes(self, historicoDosLeiloes):
        self.tabelaDeResultados.write(0, 0, "Leilo", self.estiloNegrito)
        self.tabelaDeResultados.write(0, 1, "Valor final", self.estiloNegrito)

        for i in range(len(historicoDosLeiloes)):
            self.tabelaDeResultados.write(i + 1, 0, i + 1)
            self.tabelaDeResultados.write(i + 1, 1, historicoDosLeiloes[i])

    def insereParticipante(self, participante, linha):
        self.tabelaDeResultados.write(linha, 1, participante.valorParaParticipante)
        self.tabelaDeResultados.write(linha, 2, participante.dinheiroDisponivel)
        self.tabelaDeResultados.write(linha, 3, participante.perfil)

    def salvaTabelaDeParametros(self, parametrosDoUsuario):
        self.tabelaDeParametros.write(0, 0, "Valor do produto", self.estiloNegrito)
        self.tabelaDeParametros.write(1, 0, parametrosDoUsuario.valorRealDoProduto)

        self.tabelaDeParametros.write(0, 1, "Quantidade de participantes", self.estiloNegrito)
        self.tabelaDeParametros.write(1, 1, parametrosDoUsuario.numeroDeParticipantes)

        self.tabelaDeParametros.write(0, 2, "Valor inicial", self.estiloNegrito)
        self.tabelaDeParametros.write(1, 2, parametrosDoUsuario.valorInicial)

        self.tabelaDeParametros.write(0, 3, "Aumento mnimo", self.estiloNegrito)
        self.tabelaDeParametros.write(1, 3, parametrosDoUsuario.aumentoMinimo)

        self.tabelaDeParametros.write(0, 4, "Quantidade de simulaes", self.estiloNegrito)
        self.tabelaDeParametros.write(1, 4, parametrosDoUsuario.numeroDeLeiloes)

        if parametrosDoUsuario.perfeitamenteRacionais == True:
            s = "sim"
        else:
            s = "no"
        self.tabelaDeParametros.write(0, 5, "Perfeitamente racionais", self.estiloNegrito)
        self.tabelaDeParametros.write(1, 5, s)

        self.tabelaDeParametros.write(3, 0, "Distribuio do valor para participante", self.estiloNegrito)
        self.tabelaDeParametros.write(4, 0, parametrosDoUsuario.distribuicaoValorParaParticipante)

        self.insereParametrosDasDistribuicoes(parametrosDoUsuario.distribuicaoValorParaParticipante, parametrosDoUsuario.parametrosValorParaParticipante, 5, 0)

        self.tabelaDeParametros.write(3, 1, "Distribuio do dinheiro disponvel", self.estiloNegrito)
        self.tabelaDeParametros.write(4, 1, parametrosDoUsuario.distribuicaoDinheiroDisponivel)

        self.insereParametrosDasDistribuicoes(parametrosDoUsuario.distribuicaoDinheiroDisponivel, parametrosDoUsuario.parametrosDinheiroDisponivel, 5, 1)

        if parametrosDoUsuario.perfeitamenteRacionais == False:
            self.tabelaDeParametros.write(3, 2, "Quantidade de cautelosos", self.estiloNegrito)
            self.tabelaDeParametros.write(4, 2, parametrosDoUsuario.quantidadeDeParticipantesCautelosos)

            self.tabelaDeParametros.write(3, 3, "Quantidade de moderados", self.estiloNegrito)
            self.tabelaDeParametros.write(4, 3, parametrosDoUsuario.quantidadeDeParticipantesModerados)

            self.tabelaDeParametros.write(3, 4, "Quantidade de agressivos", self.estiloNegrito)
            self.tabelaDeParametros.write(4, 4, parametrosDoUsuario.quantidadeDeParticipantesAgressivos)

    def insereParametrosDasDistribuicoes(self, distribuicao, parametros, linhaInicial, coluna):
        if distribuicao == "fixo":
            self.tabelaDeParametros.write(linhaInicial, coluna, "Valor = " + str(parametros[0]))
        elif distribuicao == "normal":
            self.tabelaDeParametros.write(linhaInicial, coluna, "Mdia = " + str(parametros[0]))
            self.tabelaDeParametros.write(linhaInicial + 1, coluna, "Varincia = " + str(parametros[1]))
        elif distribuicao == "uniforme":
            self.tabelaDeParametros.write(linhaInicial, coluna, "Incio = " + str(parametros[0]))
            self.tabelaDeParametros.write(linhaInicial + 1, coluna, "Fim = " + str(parametros[1]))
        elif distribuicao == "gamma":
            self.tabelaDeParametros.write(linhaInicial, coluna, "Alfa = " + str(parametros[0]))
            self.tabelaDeParametros.write(linhaInicial + 1, coluna, "Beta = " + str(parametros[1]))

'''
parametros = ParametrosUsuario()
parametros.valorRealDoProduto = 100
parametros.numeroDeParticipantes = 10
parametros.valorInicial = 10
parametros.aumentoMinimo = 15
parametros.numeroDeLeiloes = 1
parametros.distribuicaoDinheiroDisponivel = "gamma"
parametros.distribuicaoValorParaParticipante = "uniforme"
parametros.parametrosValorParaParticipante = [100, 100]
parametros.parametrosDinheiroDisponivel = [200, 100]
parametros.perfeitamenteRacionais = False
parametros.quantidadeDeParticipantesAgressivos = 2
parametros.quantidadeDeParticipantesCautelosos = 3
parametros.quantidadeDeParticipantesModerados = 5

leilao = Leilao(parametros)
leilao.iniciaLeilao()

nome = filesavebox(title="Selecione o arquivo", default="*.xls")
if nome != None:
    arq = SalvaXLS(nome)
    arq.salvaXLS(parametros, leilao=leilao)
'''
