#!/usr/local/bin/python
# coding: latin-1

"""
Arquivo alterado de:

Eli Bendersky (eliben@gmail.com)
License: this code is in the public domain
Last modified: 19.01.2009
"""
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, SIGNAL
from PyQt4.QtGui import QMainWindow, QFileDialog, QMessageBox, QWidget, \
    QPushButton, QCheckBox, QHBoxLayout, QApplication, QAction, QVBoxLayout, QIcon
from XLS.SalvaXLS import SalvaXLS
from classes.Leilao import Leilao
from easygui import filesavebox
from janelas.JanelaDeParametros import JanelaDeParametros
from janelas.JanelaDeResultados import JanelaDeResultados
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas, \
    NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.figure import Figure
import sys

class JanelaPrincipal(QMainWindow):
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent)
        self.setWindowTitle('Simulador de Leiles')

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("imelogo.jpg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.setWindowIcon(icon)

        self.criaMenu()
        self.criaTelaPrincipal()

        self.parametros = None

    def salvaGrafico(self):
        tiposDeArquivo = "PNG (*.png)|*.png"

        path = QFileDialog.getSaveFileName(self,
                        'Salvar arquivo', '',
                        tiposDeArquivo)

        if path:
            self.canvas.print_figure(path, dpi=self.dpi)
            self.statusBar().showMessage('Salvo em %s' % path, 2000)

    def criaSobreSimuladorDeLeiloes(self):
        msg = """ Projeto realizado por alunos do IME-USP:
        
         * Juliana Gil
         * Renata Claro
         * Thiago Silva
         
         * Orientador: Flvio Soares Corra da Silva
        """
        QMessageBox.about(self, "Simulador de Leiles", msg.strip())

    def desenhaGrade(self):
        self.axes.grid(self.grade.isChecked())
        self.canvas.draw()

    def criaTelaPrincipal(self):
        self.main_frame = QWidget()

        # Create the mpl Figure and FigCanvas objects. 
        # 5x4 inches, 100 dots-per-inch
        #
        self.dpi = 100
        self.fig = Figure((8.0, 5.0), dpi=self.dpi)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)

        # Since we have only one plot, we can use add_axes 
        # instead of add_subplot, but then the subplot
        # configuration tool in the navigation toolbar wouldn't
        # work.
        #
        self.axes = self.fig.add_subplot(111)

        # Create the navigation toolbar, tied to the canvas
        #
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls
        #         
        self.botaoConfigurarLeilao = QPushButton("&Configurar Leilo")
        self.connect(self.botaoConfigurarLeilao, SIGNAL('clicked()'), self.abreJanelaDeParametros)

        self.botaoVerResultados = QPushButton("Ver &Resultados")
        self.botaoVerResultados.setEnabled(False)
        self.connect(self.botaoVerResultados, SIGNAL('clicked()'), self.abreJanelaDeResultados)

        self.botaoSalvarResultados = QPushButton("&Salvar Resultados")
        self.botaoSalvarResultados.setEnabled(False)
        self.connect(self.botaoSalvarResultados, SIGNAL('clicked()'), self.salvaResultados)

        self.grade = QCheckBox("Mostrar &Grade")
        self.grade.setChecked(False)
        self.connect(self.grade, SIGNAL('stateChanged(int)'), self.desenhaGrade)

        #
        # Layout with box sizers
        # 
        hbox = QHBoxLayout()

        for w in [self.grade, self.botaoConfigurarLeilao, self.botaoVerResultados, self.botaoSalvarResultados]:
            hbox.addWidget(w)
            hbox.setAlignment(w, Qt.AlignVCenter)

        vbox = QVBoxLayout()
        vbox.addWidget(self.canvas)
        vbox.addWidget(self.mpl_toolbar)
        vbox.addLayout(hbox)

        self.main_frame.setLayout(vbox)
        self.setCentralWidget(self.main_frame)

    def criaMenu(self):
        self.menuArquivo = self.menuBar().addMenu("&Arquivo")

        acaoConfigurarLeilao = self.criaAcao("&Configurar Leilo",
            shortcut="Ctrl+C", slot=self.abreJanelaDeParametros)
        acaoSair = self.criaAcao("&Sair", slot=self.close,
            shortcut="Ctrl+Q")

        self.adicionaAcao(self.menuArquivo,
            (acaoConfigurarLeilao, None, acaoSair))

        self.menuAjuda = self.menuBar().addMenu("&Ajuda")
        about_action = self.criaAcao("Sobre o Simulador",
            shortcut='F1', slot=self.criaSobreSimuladorDeLeiloes)

        self.adicionaAcao(self.menuAjuda, (about_action,))

    def adicionaAcao(self, target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)

    def criaAcao(self, text, slot=None, shortcut=None,
                        icon=None, tip=None, checkable=False,
                        signal="triggered()"):
        action = QAction(text, self)
        if icon is not None:
            action.setIcon(QIcon(":/%s.png" % icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            self.connect(action, SIGNAL(signal), slot)
        if checkable:
            action.setCheckable(True)

        return action

    def abreJanelaDeParametros(self):
        self.dialogParametros = QtGui.QDialog()
        self.janelaDeParametros = JanelaDeParametros(self, self.parametros)
        self.janelaDeParametros.setupUi(self.dialogParametros)
        self.dialogParametros.show()

    def abreJanelaDeResultados(self):
        numeroDeLeiloes = self.parametros.numeroDeLeiloes
        if numeroDeLeiloes == 1:
            self.janelaDeResultados = JanelaDeResultados("Participante", "Lance", len(self.historicoDosLances), 1, autoresDosLances=self.autoresDosLances, historicoDosLances=self.historicoDosLances)
        else:
            self.janelaDeResultados = JanelaDeResultados("Leilo", "Valor Final", len(self.resultadoDosLeiloes), numeroDeLeiloes, resultadoDosLeiloes=self.resultadoDosLeiloes)

        self.dialogResultados = QtGui.QDialog()
        self.janelaDeResultados.setupUi(self.dialogResultados)
        self.dialogResultados.show()

    def setParametros(self, parametros):
        self.parametros = parametros

    def iniciaLeiloes(self):
        self.dialogParametros.close()

        self.resultadoDosLeiloes = list()

        i = 0
        media = 0
        while i < self.parametros.numeroDeLeiloes:
            leilao = Leilao(self.parametros)
            lances, autor = leilao.iniciaLeilao()

            ultimoIndice = len(lances) - 1
            if self.parametros.leilaoIngles == False and len(lances) >= 2:
                self.resultadoDosLeiloes.append(lances[ultimoIndice - 1])
            else:
                self.resultadoDosLeiloes.append(lances[ultimoIndice])

            i += 1

            media += lances[ultimoIndice - 1]

        media = media / self.parametros.numeroDeLeiloes
        print (media)

        if self.parametros.numeroDeLeiloes == 1:
            self.historicoDosLances = lances
            self.autoresDosLances = autor
            self.leilao = leilao

        self.desenhaGrafico()

        #libera para salvar e ver resultados
        self.botaoSalvarResultados.setEnabled(True)
        self.botaoVerResultados.setEnabled(True)

    def desenhaGrafico(self):
        self.axes.clear()

        if self.parametros.numeroDeLeiloes == 1:
            tempo = range(1, len(self.historicoDosLances) + 1)

            self.axes.plot(tempo, self.historicoDosLances, 'k')
            self.axes.plot(tempo, self.historicoDosLances, 'bo')

            if self.parametros.leilaoIngles == False and len(tempo) >= 2 and len(self.historicoDosLances) >= 2:
                self.axes.plot(tempo[-2], self.historicoDosLances[-2], 'ro')
        else:
            self.axes.hist(self.resultadoDosLeiloes)

        self.canvas.draw()

    def salvaResultados(self):
        salva = SalvaXLS(filesavebox(title="Selecione onde deseja salvar", default="*.xls"))

        if self.parametros.numeroDeLeiloes == 1:
            salva.salvaXLS(self.parametros, leilao=self.leilao)
        else:
            salva.salvaXLS(self.parametros, historicoDosLeiloes=self.resultadoDosLeiloes)

def main():
    app = QApplication(sys.argv)
    form = JanelaPrincipal()
    form.show()
    app.exec_()


if __name__ == "__main__":
    main()
