#!/usr/local/bin/python
# coding: latin-1

from classes.ParametrosUsuario import ParametrosUsuario
import unittest
from XLS.SalvaXLS import SalvaXLS
from xlrd import open_workbook
from classes.Leilao import Leilao

class TesteSalvaXLS(unittest.TestCase):
    def setUp(self):
        self.parametrosParaPerfeitamenteRacionais = ParametrosUsuario()
        self.parametrosParaPerfeitamenteRacionais.distribuicaoDinheiroDisponivel = "fixo"
        self.parametrosParaPerfeitamenteRacionais.distribuicaoValorParaParticipante = "normal"
        self.parametrosParaPerfeitamenteRacionais.numeroDeLeiloes = 1
        self.parametrosParaPerfeitamenteRacionais.numeroDeParticipantes = 10
        self.parametrosParaPerfeitamenteRacionais.parametrosDinheiroDisponivel = [2000]
        self.parametrosParaPerfeitamenteRacionais.parametrosValorParaParticipante = [400, 100]
        self.parametrosParaPerfeitamenteRacionais.perfeitamenteRacionais = True
        self.parametrosParaPerfeitamenteRacionais.valorRealDoProduto = 500
        self.parametrosParaPerfeitamenteRacionais.valorInicial = 50
        self.parametrosParaPerfeitamenteRacionais.aumentoMinimo = 25

        self.parametrosParaPerfisDiferentes = ParametrosUsuario()
        self.parametrosParaPerfisDiferentes.distribuicaoDinheiroDisponivel = "gamma"
        self.parametrosParaPerfisDiferentes.distribuicaoValorParaParticipante = "uniforme"
        self.parametrosParaPerfisDiferentes.numeroDeLeiloes = 1
        self.parametrosParaPerfisDiferentes.numeroDeParticipantes = 10
        self.parametrosParaPerfisDiferentes.parametrosDinheiroDisponivel = [400, 200]
        self.parametrosParaPerfisDiferentes.parametrosValorParaParticipante = [200, 1000]
        self.parametrosParaPerfisDiferentes.perfeitamenteRacionais = False
        self.parametrosParaPerfisDiferentes.valorRealDoProduto = 500
        self.parametrosParaPerfisDiferentes.valorInicial = 50
        self.parametrosParaPerfisDiferentes.aumentoMinimo = 25
        self.parametrosParaPerfisDiferentes.quantidadeDeParticipantesAgressivos = 3
        self.parametrosParaPerfisDiferentes.quantidadeDeParticipantesCautelosos = 2
        self.parametrosParaPerfisDiferentes.quantidadeDeParticipantesModerados = 5

    def testeSalvaTabelaDeParametrosParaPerfeitamenteRacionais(self):
        salva = SalvaXLS("testeSalvaTabelaDeParametrosParaPerfeitamenteRacionais.xls")
        salva.salvaTabelaDeParametros(self.parametrosParaPerfeitamenteRacionais)
        salva.arquivo.save(salva.nomeDoArquivo)

        arquivoSalvo = open_workbook("testeSalvaTabelaDeParametrosParaPerfeitamenteRacionais.xls")
        tabela = arquivoSalvo.sheet_by_index(0)

        self.assertEqual(tabela.cell(1, 0).value, self.parametrosParaPerfeitamenteRacionais.valorRealDoProduto)
        self.assertEqual(tabela.cell(1, 1).value, self.parametrosParaPerfeitamenteRacionais.numeroDeParticipantes)
        self.assertEqual(tabela.cell(1, 2).value, self.parametrosParaPerfeitamenteRacionais.valorInicial)
        self.assertEqual(tabela.cell(1, 3).value, self.parametrosParaPerfeitamenteRacionais.aumentoMinimo)
        self.assertEqual(tabela.cell(1, 4).value, self.parametrosParaPerfeitamenteRacionais.numeroDeLeiloes)
        self.assertEqual(tabela.cell(1, 5).value, "sim")
        self.assertEqual(tabela.cell(4, 0).value, self.parametrosParaPerfeitamenteRacionais.distribuicaoValorParaParticipante)
        self.assertEqual(tabela.cell(4, 1).value, self.parametrosParaPerfeitamenteRacionais.distribuicaoDinheiroDisponivel)

    def testeSalvaTabelaDeParametrosParaPerfisDiferentes(self):
        salva = SalvaXLS("testeSalvaTabelaDeParametrosParaPerfisDiferentes.xls")
        salva.salvaTabelaDeParametros(self.parametrosParaPerfisDiferentes)
        salva.arquivo.save(salva.nomeDoArquivo)

        arquivoSalvo = open_workbook("testeSalvaTabelaDeParametrosParaPerfisDiferentes.xls")
        tabela = arquivoSalvo.sheet_by_index(0)

        self.assertEqual(tabela.cell(1, 0).value, self.parametrosParaPerfisDiferentes.valorRealDoProduto)
        self.assertEqual(tabela.cell(1, 1).value, self.parametrosParaPerfisDiferentes.numeroDeParticipantes)
        self.assertEqual(tabela.cell(1, 2).value, self.parametrosParaPerfisDiferentes.valorInicial)
        self.assertEqual(tabela.cell(1, 3).value, self.parametrosParaPerfisDiferentes.aumentoMinimo)
        self.assertEqual(tabela.cell(1, 4).value, self.parametrosParaPerfisDiferentes.numeroDeLeiloes)
        self.assertEqual(tabela.cell(1, 5).value, "no")
        self.assertEqual(tabela.cell(4, 0).value, self.parametrosParaPerfisDiferentes.distribuicaoValorParaParticipante)
        self.assertEqual(tabela.cell(4, 1).value, self.parametrosParaPerfisDiferentes.distribuicaoDinheiroDisponivel)
        self.assertEqual(tabela.cell(4, 2).value, self.parametrosParaPerfisDiferentes.quantidadeDeParticipantesCautelosos)
        self.assertEqual(tabela.cell(4, 3).value, self.parametrosParaPerfisDiferentes.quantidadeDeParticipantesModerados)
        self.assertEqual(tabela.cell(4, 4).value, self.parametrosParaPerfisDiferentes.quantidadeDeParticipantesAgressivos)

    def testeSalvaXLSParaUmLeilao(self):
        leilao = Leilao(self.parametrosParaPerfisDiferentes)
        leilao.iniciaLeilao()

        salva = SalvaXLS("testeSalvaXLSParaUmLeilao.xls")
        salva.salvaXLS(self.parametrosParaPerfisDiferentes, leilao=leilao)

        arquivoSalvo = open_workbook("testeSalvaXLSParaUmLeilao.xls")
        tabela = arquivoSalvo.sheet_by_name("Resultados")

        for i in range(leilao.numeroDeParticipantes):
            participante = leilao.participantes[i]
            self.assertEqual(tabela.cell(i + 1, 1).value, participante.valorParaParticipante)
            self.assertEqual(tabela.cell(i + 1, 2).value, participante.dinheiroDisponivel)
            self.assertEqual(tabela.cell(i + 1, 3).value, participante.perfil)

        for i in range(len(leilao.historicoDosLances)):
            self.assertEqual(tabela.cell(i + 1, 5).value, leilao.autorDoLance[i] + 1)
            self.assertEqual(tabela.cell(i + 1, 6).value, leilao.historicoDosLances[i])
